using FlexCel.Core;
using FlexCel.Pdf;
using FlexCel.Render;
using FlexCel.Report;
using FlexCel.XlsAdapter;
using System.Globalization;
using System.Reflection;
using Microsoft.Data.Sqlite;

namespace AndroidFlexcelTestApp
{
   [Activity(Label = "@string/app_name", MainLauncher = true)]
   public class MainActivity : Activity
   {
      protected override void OnCreate(Bundle? savedInstanceState)
      {
         base.OnCreate(savedInstanceState);

         // Set our view from the "main" layout resource
         SetContentView(Resource.Layout.activity_main);

         Button coldstartbutton = FindViewById<Button>(Resource.Id.coldstartbutton);
         coldstartbutton.Click += ColdStartButtonOnClick;
      }

      private void ColdStartButtonOnClick(object sender, EventArgs eventArgs)
      {
         //Thread.CurrentThread.CurrentCulture = new CultureInfo("en");
         Thread.CurrentThread.CurrentCulture = new CultureInfo("sk");
         Thread.CurrentThread.CurrentUICulture = Thread.CurrentThread.CurrentCulture;
         ColdStart();
      }


      private void ColdStart()
      {
         Assembly assembly = Assembly.GetExecutingAssembly();
         bool startedSuccessfully = true;
         using (Stream coldStartTemplateStream = assembly.GetManifestResourceStream("AndroidFlexcelTestApp.Resources.coldstarttemplate.xlsx"))
         {
            using (MemoryStream flexCelReportResultStream = new MemoryStream())
            {
               XlsFile xls = null;
               try
               {
                  using (FlexCelReport fr = new FlexCelReport(true))
                  {
                     using (SqliteConnection connection = new SqliteConnection("Data Source=:memory:"))
                     {
                        fr.DisableSQLValidation = true;
                        connection.Open();
                        SQLiteDataAdapter adapter = PrepareColdStartDatabase(connection);
                        fr.AddConnection("db", adapter, CultureInfo.InvariantCulture);
                        fr.Run(coldStartTemplateStream, flexCelReportResultStream);
                        connection.Close();
                     }
                     flexCelReportResultStream.Position = 0;

                     xls = new XlsFile(true);
                     string pathToSave = Path.Combine(GetExternalFilesDir(null).AbsolutePath, "result.xlsx");
                     xls.Open(flexCelReportResultStream);
                     xls.Save(pathToSave);
                  }
               }
               catch (Exception e)
               {
                  startedSuccessfully = false;
               }

               if (xls != null && startedSuccessfully)
               {
                  startedSuccessfully = MakeInMemoryPdf(xls);
               }
            }
         }

         if (startedSuccessfully)
         {
         }
      }

      private SQLiteDataAdapter PrepareColdStartDatabase(SqliteConnection connection)
      {
         SqliteCommand createCommand = new SqliteCommand("CREATE TABLE _dbinfo(DB_VERSION NUMERIC(17,4), CREATED_DATE DATETIMEOFFSET, DELIVERY_DATE DATETIME);", connection);
         createCommand.ExecuteNonQuery();
         SqliteCommand insertCommand = new SqliteCommand("INSERT INTO _dbinfo(DB_VERSION, CREATED_DATE, DELIVERY_DATE) VALUES(1782.97, '2014-10-01 00:00:00+01:00', '28-06-2015');", connection);
         insertCommand.ExecuteNonQuery();
         return new SQLiteDataAdapter(String.Empty, connection);
      }

      private bool MakeInMemoryPdf(XlsFile xls)
      {
         FlexCelConfig.GraphicFramework = FlexCelGraphicFramework.Native;
         FlexCelTrace.OnError += HandleOnFlexcelError;
         bool result = true;
         using (MemoryStream pdfStream = new MemoryStream())
         {
            using (FlexCelPdfExport pdf = new FlexCelPdfExport(xls, true))
            {
               try
               {

                  string pathToSave = Path.Combine(GetExternalFilesDir(null).AbsolutePath, "result.pdf");
                  pdf.FontMapping = TFontMapping.ReplaceAllFonts;
                  pdf.GetFontData += HandleOnGetFontData;
                  pdf.BeforeGeneratePage += HandleOnBeforeGeneratingPage;
                  pdf.OnFontFolderNotFound = TPdfFontFolderNotFoundAction.Ignore;
                  pdf.UnlicensedFontAction = TUnlicensedFontAction.Ignore;
                  pdf.AfterGeneratePage += HandleOnAfterGeneratingPage;
                  pdf.GetFontFolder += HandleOnGetFontFolder;
                 
                  pdf.ExportAllVisibleSheets(pathToSave,true,null);
                  result = true;
               }

               catch (Exception e)
               {
                  result = false;
               }
            }
         }

         return result;
      }

      private void HandleOnAfterGeneratingPage(object sender, PageEventArgs e)
      {
         
      }

      private void HandleOnGetFontFolder(object sender, GetFontFolderEventArgs e)
      {
      }

      private void HandleOnBeforeGeneratingPage(object sender, PageEventArgs e)
      {
      }

      private void HandleOnGetFontData(object sender, GetFontDataEventArgs e)
      {
      }

      private void HandleOnFlexcelError(TFlexCelErrorInfo e)
      {
      }
   }
}